#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	mat3 TBN;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightColor;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform vec3 matColor;

uniform bool useTexture = false;
uniform sampler2D diffuseTexture;
uniform bool useSpecularMap = false;
uniform sampler2D specularMap;
uniform bool useNormalMap = false;
uniform sampler2D normalMap;


uniform samplerCube cubemap;
uniform float reflectionFactor = 1.0f; 



void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	if(useNormalMap){
		n = texture(normalMap, vert.uv).rgb;
		n = normalize(vert.TBN * n);
	}
	
	vec3 texColor;
	if(useTexture)
		texColor = texture(diffuseTexture, vert.uv).rgb;
	else
		texColor = matColor;
	
	color = vec4(texColor * materialCoefficients.x, 1); // ambient


	color.rgb *= (1.0f - reflectionFactor);
	vec3 reflection = reflect(-v, n);
	color.rgb += texture(cubemap, reflection).rgb * reflectionFactor;


	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
	if(brightness > 0.8)
        brightColor = vec4(color.rgb, 1.0);
    else
        brightColor = vec4(0.0, 0.0, 0.0, 1.0);
}

